<?php
/**
 * weDevs Settings API wrapper class
 *
 * @author Tareq Hasan <tareq@weDevs.com>
 * @link http://tareq.weDevs.com Tareq's Planet
 * @example settings-api.php How to use the class
 */
if (!class_exists('CbtwitterCard_Settings_API')):

    class CbtwitterCard_Settings_API {

        /**
         * settings sections array
         *
         * @var array
         */
        private $settings_sections = array();

        /**
         * Settings fields array
         *
         * @var array
         */
        private $settings_fields = array();

        /**
         * Singleton instance
         *
         * @var object
         */
        private static $_instance;

        public function __construct() {

            add_action('admin_enqueue_scripts', array($this, 'admin_enqueue_scripts'));
            // add_action( 'admin_init', 'wedevs_admin_init' );
        }

        /**
         * Enqueue scripts and styles
         */
        function admin_enqueue_scripts() {
            //loads thickbox
            add_thickbox();

            wp_enqueue_style('wp-color-picker');
            wp_enqueue_style('thickbox');

            wp_enqueue_script('jquery');
            wp_enqueue_script('wp-color-picker');
            wp_enqueue_script('media-upload');
            wp_enqueue_script('thickbox');
        }

        /**
         * Set settings sections
         *
         * @param array   $sections setting sections array
         */
        function set_sections($sections) {

            $this->settings_sections = $sections;

            return $this;
        }

        /**
         * Add a single section
         *
         * @param array   $section
         */
        function add_section($section) {

            $this->settings_sections[] = $section;

            return $this;
        }

        /**
         * Set settings fields
         *
         * @param array   $fields settings fields array
         */
        function set_fields($fields) {
            $this->settings_fields = $fields;

            return $this;
        }

        function add_field($section, $field) {
            $defaults = array(
                'name'  => '',
                'label' => '',
                'desc'  => '',
                'type'  => 'text'
            );

            $arg                               = wp_parse_args($field, $defaults);
            $this->settings_fields[$section][] = $arg;
            //var_dump($this->settings_fields[$section]);

            return $this;
        }

        /**
         * Initialize and registers the settings sections and fileds to WordPress
         *
         * Usually this should be called at `admin_init` hook.
         *
         * This function gets the initiated settings sections and fields. Then
         * registers them to WordPress and ready for use.
         */
        function admin_init() {
            //register settings sections

            foreach ($this->settings_sections as $section) {
                if (false == get_option($section['id'])) {
                    add_option($section['id']);
                }

                if (isset($section['desc']) && !empty($section['desc'])) {
                    $section['desc'] = '<div class="inside">' . $section['desc'] . '</div>';
                    $callback        = create_function('', 'echo "' . str_replace('"', '\"', $section['desc']) . '";');
                } else {
                    $callback = '__return_false';
                }

                add_settings_section($section['id'], $section['title'], $callback, $section['id']);
            }

            //register settings fields
            foreach ($this->settings_fields as $section => $field) {
                foreach ($field as $option) {

                    $type = isset($option['type']) ? $option['type'] : 'text';

                    $args = array(
                        'id'                => $option['name'],
                        'desc'              => isset($option['desc']) ? $option['desc'] : '',
                        'name'              => $option['label'],
                        'section'           => $section,
                        'beforeinput'       => array_key_exists('beforeinput', $option) ? $option['beforeinput'] : '',
                        'placeholder'       => array_key_exists('placeholder', $option) ? $option['placeholder'] : '',
                        'tooltip'           => array_key_exists('tooltip', $option) ? $option['tooltip'] : '',
                        'size'              => isset($option['size']) ? $option['size'] : null,
                        'options'           => isset($option['options']) ? $option['options'] : '',
                        'std'               => isset($option['default']) ? $option['default'] : '',
                        'sanitize_callback' => isset($option['sanitize_callback']) ? $option['sanitize_callback'] : '',
                        'preview'           => isset($option['preview']) ? $option['preview'] : false,
                    );

                    add_settings_field($section . '[' . $option['name'] . ']', $option['label'], array($this, 'callback_' . $type), $section, $section, $args);
                }
            }

            // creates our settings in the options table
            foreach ($this->settings_sections as $section) {
                register_setting($section['id'], $section['id'], array($this, 'sanitize_options'));
            }
        }

        /**
         * Displays a text field for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_text($args) {
            $cbtc_before_input = '';
            if (array_key_exists('beforeinput', $args) && $args['beforeinput'] == 'on') {
                $cbtc_before_input = '@';
            }
            // var_dump($args);
            $value = esc_attr($this->get_option($args['id'], $args['section'], $args['std']));
            $size  = isset($args['size']) && !is_null($args['size']) ? $args['size'] : 'regular';
            //do_action('cbtwittercard_before_text');
            $html  = $cbtc_before_input;
            $html .= sprintf('<input type="text" style="height: 30px;margin-bottom: 5px;" class="%3$s %1$s-text" id="%2$s[%3$s]" name="%2$s[%3$s]" value="%4$s"/>', $size, $args['section'], $args['id'], $value);
            if (array_key_exists('tooltip', $args) && $args['tooltip'] != '') {
                $cbtc_tooltip = '<span title="' . $args["tooltip"] . '" class="cbtwittercard_tooltip"><img src="' . CBTWITTERCARDTOOLTIP . '"/></span>';
            } else {
                $cbtc_tooltip = '';
            }
            $html .= sprintf('%1$s<br><span class="description"> %2$s</span>', $cbtc_tooltip, $args['desc']);

            echo $html;
        }

        /**
         * @param $args
         * Displays a number field for a settings field
         */
        function callback_number($args) {

            $value = esc_attr($this->get_option($args['id'], $args['section'], $args['std']));
            $size  = isset($args['size']) && !is_null($args['size']) ? $args['size'] : 'regular';
            $html  = sprintf('<input type="number" style="height: 30px;margin-bottom: 5px;" class="%1$s-text" id="%2$s-%3$s" name="%2$s[%3$s]" value="%4$s"/>', $size, $args['section'], $args['id'], $value);
            if (array_key_exists('tooltip', $args) && $args['tooltip'] != '') {
                $cbtc_tooltip = '<span title="' . $args["tooltip"] . '" class="cbtwittercard_tooltip"><img src="' . CBTWITTERCARDTOOLTIP . '"/></span>';
            } else {
                $cbtc_tooltip = '';
            }
            $html .= sprintf('%1$s<br><span class="description"> %2$s</span>', $cbtc_tooltip, $args['desc']);


            echo $html;
        }

        /**
         * @param $args
         * displays repeated field now only product
         * added by codeboxr
         */
        function callback_repeat($args) {


            $value = get_option($args['section']);

            if ($value != '') {

                $value_of_ids = ($value[$args['id']]);
                $html         = '';
                $html .= '<div class="cbtwittercard-repeatfields-wrapper">';

                foreach ($args['options'] as $index => $option) {
                    $html .= '<input class="cbtwittercard-repeatfields-groups" type="hidden" value="' . $option['name'] . '"/>';
                }

                $countlabel = 'count';
                $html .= '<input class="cbtwittercard-repeatfields-count" type="hidden" name="' . $args['section'] . '[' . $args['id'] . '][' . $countlabel . ']" value="' . $value_of_ids['count'] . '"/><ul class="cbtwittercard-repeatfields">';

                if (is_array($value_of_ids) && !empty($value_of_ids)) {

                    for ($i = 0; $i < $value_of_ids['count']; $i++) {

                        $html .= '<li>';
                        foreach ($args['options'] as $index => $option) {
                            $value = $value_of_ids[$option['name']][$i];
                            $html .= '<input type="text" style="width:100px;height: 30px;margin-bottom: 5px;" class="regular-text ' . $option['name'] . '" id="' . $args['section'] . $args['id'] . $option['name'] . '[' . $i . ']" name="' . $args['section'] . '[' . $args['id'] . '][' . $option['name'] . '][]" value="' . $value . '"/>';
                        }
                        $html .= '<a href="#" class="button cbtwittercard-remove">' . __('Remove', 'cbtwittercard') . '</a>';
                        $html .= '</li>';
                    }
                }

                $html .= '</ul><span class="description">' . $args['desc'] . ' </span></div>';
                $html .= '<a href="#" data-section = "' . $args['section'] . '" data-count = "' . $value_of_ids['count'] . '" data-id = "' . $args['id'] . '" class=" button cbtwittercard-add-new">Add New </a>';
                echo $html;
            } else {
                $html = '';

                $html .= '<div class="cbtwittercard-repeatfields-wrapper">';

                foreach ($args['options'] as $index => $option) {
                    $html .= '<input class="cbtwittercard-repeatfields-groups" type="hidden" value="' . $option['name'] . '"/>';
                }
                $countlabel = 'count';
                $html .= '<input class="cbtwittercard-repeatfields-count" type="hidden" name="' . $args['section'] . '[' . $args['id'] . '][' . $countlabel . ']" value="1"/><ul class="cbtwittercard-repeatfields"><li>';

                foreach ($args['options'] as $index => $option) {
                    $html .= '<input type="text" style="width:100px;height: 30px;margin-bottom: 5px;" class="regular-text ' . $option['name'] . '" id="' . $args['section'] . $args['id'] . $option['name'] . '[0]" name="' . $args['section'] . '[' . $args['id'] . '][' . $option['name'] . '][]" value=" "/>';
                }
                $html .= '<a href="#" class="button cbtwittercard-remove">Remove</a></li></ul><span class="description">' . $args['desc'] . ' </span></div>';
                $html .= '<a href="#" data-section = "' . $args['section'] . '" data-id = "' . $args['id'] . '" data-count = "1" class=" button cbtwittercard-add-new">Add New </a>';
                echo $html;
            }
        }

        /**
         * @param $args
         * simple version of callback no remove
         */
        function callback_repeatsimple($args) {


            $value = get_option($args['section']);

            if ($value != '') {

                $value_of_ids = ($value[$args['id']]);

                $html = '';
                $html .= '<div class="cbtwittercard-repeatfields-wrapper"><ul class="cbtwittercard-repeatfields">';
                if (is_array($value_of_ids) && !empty($value_of_ids)) {
                    $fields_in_group_count = '';
                    foreach ($value_of_ids as $id => $values) {
                        // var_dump($id);
                        $fields_in_group_count = count($value_of_ids[$id]);
                        $html .= '<input class="cbtwittercard-repeatfields-groups" type="hidden" value="' . $id . '"/>';
                        foreach ($values as $index => $value) {

                            $html .= '<li><input type="text" style="height: 30px;margin-bottom: 5px;" class="regular-text ' . $id . '" id="' . $args['section'] . $args['id'] . $id . '[' . $index . ']" name="' . $args['section'] . '[' . $args['id'] . '][' . $id . '][]" value="' . $value . '"/></li>';
                        }
                    }
                }

                $html .= '</ul><span class="description">' . $args['desc'] . ' </span></div>';
                $html .= '<a href="#" data-section = "' . $args['section'] . '" data-count = "' . $fields_in_group_count . '" data-id = "' . $args['id'] . '" class=" button cbtwittercard-add-new">Add New </a>';
                echo $html;
            } else {
                $html = '';
                $html .= '<div class="cbtwittercard-repeatfields-wrapper"><ul class="cbtwittercard-repeatfields">';

                foreach ($args['options'] as $index => $option) {

                    $html .= '<input class="cbtwittercard-repeatfields-groups" type="hidden" value="' . $option['name'] . '"/>';
                    $html .= '<li><input type="text" style="height: 30px;margin-bottom: 5px;" class="regular-text ' . $option['name'] . '" id="' . $args['section'] . $args['id'] . $option['name'] . '[0]" name="' . $args['section'] . '[' . $args['id'] . '][' . $option['name'] . '][]" value=" "/></li>';
                }
                $html .= '</ul><span class="description">' . $args['desc'] . ' </span></div>';
                $html .= '<a href="#" data-section = "' . $args['section'] . '" data-id = "' . $args['id'] . '" data-count = "1" class=" button cbtwittercard-add-new">Add New </a>';
                echo $html;
            }
        }

        /**
         * Displays a checkbox for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_checkbox($args) {

            $value = esc_attr($this->get_option($args['id'], $args['section'], $args['std']));
            $html  = sprintf('<input type="hidden" name="%1$s[%2$s]" value="off" />', $args['section'], $args['id']);
            $html .= sprintf('<input type="checkbox" class="checkbox" id="%1$s[%2$s]" name="%1$s[%2$s]" value="on"%4$s />', $args['section'], $args['id'], $value, checked($value, 'on', false));
            $html .= sprintf('<span for="%1$s[%2$s]"> %3$s</span>', $args['section'], $args['id'], $args['desc']);

            echo $html;
        }

        /**
         * Displays a multicheckbox a settings field
         *
         * @param array   $args settings field args
         */
        function callback_multicheck($args) {

            $value = $this->get_option($args['id'], $args['section'], $args['std']);
            $html  = '';

            foreach ($args['options'] as $key => $label) {

                $checked = isset($value[$key]) ? $value[$key] : '0';
                $html .= sprintf('<input type="checkbox" class="checkbox " id="%1$s[%2$s][%3$s]" name="%1$s[%2$s][%3$s]" value="%3$s"%4$s />', $args['section'], $args['id'], $key, checked($checked, $key, false));
                $html .= sprintf('<span for="%1$s[%2$s][%4$s]"> %3$s</span><br>', $args['section'], $args['id'], $label, $key);
            }
            $html .= sprintf('<span class="description"> %s</label>', $args['desc']);

            echo $html;
        }

        /**
         * @param $args
         * a multi select box added by codeboxr
         *
         */
        function callback_postselectbox($args) {

            $value = $this->get_option($args['id'], $args['section'], $args['std']);
            //var_dump($value);

            $html = '<div class="text">';

            foreach ($args['options'] as $key => $label) {

                $checked = isset($value[$key]) ? $value[$key] : '0';
                //var_dump($checked);
                $html .= sprintf('<input type="checkbox" class="checkbox multi-select" style="display:none" id="%1$s[%2$s][%3$s]" name="%1$s[%2$s][%3$s]" value="%3$s" %4$s />', $args['section'], $args['id'], $key, checked($checked, $key, false));
                $html .= sprintf('<label for="%1$s[%2$s][%4$s]" style="display:none"> %3$s</label>', $args['section'], $args['id'], $label, $key);
            }

            $html .= '<select multiple="yes" style="width: 200px;" class="multiselect chosen-select" >';

            foreach ($args['options'] as $key => $label) {

                $checked = isset($value[$key]) ? $value[$key] : '0';
                $html .= sprintf('<option  id="%1$s[%2$s][%3$s]" name="%1$s[%2$s][%3$s]" value="%3$s" %4$s >
            ', $args['section'], $args['id'], $key, selected($checked, $key, false));
                $html .= sprintf('<span for="%1$s[%2$s][%4$s]"> %3$s</span>', $args['section'], $args['id'], $label, $key);
                $html .='</option>';
            }
            $html .= sprintf('</select><span class="description"> %s</label></div>', $args['desc']);

            echo $html;
        }

        /**
         * Displays a multicheckbox a settings field
         *
         * @param array   $args settings field args
         */
        function callback_radio($args) {

            $value = $this->get_option($args['id'], $args['section'], $args['std']);
            $html  = '';

            foreach ($args['options'] as $key => $label) {

                $html .= sprintf('<input type="radio" class="radio" id="%1$s[%2$s][%3$s]" name="%1$s[%2$s]" value="%3$s"%4$s />', $args['section'], $args['id'], $key, checked($value, $key, false));
                $html .= sprintf('<span for="%1$s[%2$s][%4$s]"> %3$s</span><br>', $args['section'], $args['id'], $label, $key);
            }
            $html .= sprintf('<span class="description"> %s</label>', $args['desc']);

            echo $html;
        }

        /**
         * Displays a selectbox for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_select($args) {

            $value = esc_attr($this->get_option($args['id'], $args['section'], $args['std']));
            $size  = isset($args['size']) && !is_null($args['size']) ? $args['size'] : 'regular';
            $html  = sprintf('<select class="%1$s"  name="%2$s[%3$s]" id="%2$s[%3$s]">', $size, $args['section'], $args['id']);

            foreach ($args['options'] as $key => $label) {

                $html .= sprintf('<option value="%s"%s>%s</option>', $key, selected($value, $key, false), $label);
            }
            $html .= sprintf('</select>');
            $html .= sprintf('<span class="description"> %s</span>', $args['desc']);

            echo $html;
        }

        /**
         * Displays a textarea for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_textarea($args) {

            $value = esc_textarea($this->get_option($args['id'], $args['section'], $args['std']));
            $size  = isset($args['size']) && !is_null($args['size']) ? $args['size'] : 'regular';

            $html = sprintf('<textarea style="width: 350px;" rows="5" cols="155" class="%3$s %1$s-text" id="%2$s[%3$s]" name="%2$s[%3$s]">%4$s</textarea>', $size, $args['section'], $args['id'], $value);
            if (array_key_exists('tooltip', $args) && $args['tooltip'] != '') {
                $cbtc_tooltip = '<span title="' . $args["tooltip"] . '" class="cbtwittercard_tooltip"><img src="' . CBTWITTERCARDTOOLTIP . '"/></span>';
            } else {
                $cbtc_tooltip = '';
            }
            $html .= sprintf('%1$s', $cbtc_tooltip);
            $html .= sprintf('<br><span class="description"> %s</span>', $args['desc']);

            echo $html;
        }

        /**
         * Displays a textarea for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_html($args) {
            echo $args['desc'];
        }

        /**
         * Displays a rich text textarea for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_wysiwyg($args) {

            $value = wpautop($this->get_option($args['id'], $args['section'], $args['std']));
            $size  = isset($args['size']) && !is_null($args['size']) ? $args['size'] : '500px';

            echo '<div style="width: ' . $size . ';">';

            wp_editor($value, $args['section'] . '[' . $args['id'] . ']', array('teeny' => true, 'textarea_rows' => 10));

            echo '</div>';

            echo sprintf('<br><span class="description"> %s</span>', $args['desc']);
        }

        /**
         * Displays a file upload field for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_file($args) {

            //var_dump($args);

            $value = esc_attr($this->get_option($args['id'], $args['section'], $args['std']));

            $size   = isset($args['size']) && !is_null($args['size']) ? $args['size'] : 'regular';
            $id     = $args['section'] . '[' . $args['id'] . ']';
            $html   = sprintf('<input type="text" class="%1$s-text wpsa-url" id="%2$s-%3$s" name="%2$s[%3$s]" value="%4$s"/>', $size, $args['section'], $args['id'], $value);
            $dataid = $args['section'] . '-' . $args['id'];

            $preview_html = '';
            //show image preview is set
            if (isset($args['preview']) && $args['preview']) {
                if ($value != '') {
                    $preview_html = '<br/><span class="cbtwittercard-wpsa-review" id="' . $dataid . '_review" ><img style="width:200px; height:200px;" src="' . $value . '" /></span>';
                } else {
                    $preview_html = '<br/><span class="cbtwittercard-wpsa-review" id="' . $dataid . '_review" ></span>';
                }
            }






            $html .= '<input type="button" data-id ="' . $dataid . '" class="button cbtwittercard-wpsa-browse" value="' . __('Browse') . '" />';

            if (array_key_exists('tooltip', $args) && $args['tooltip'] != '') {
                $cbtc_tooltip = '<span title="' . $args["tooltip"] . '" class="cbtwittercard_tooltip"><img src="' . CBTWITTERCARDTOOLTIP . '"/></span>';
            } else {
                $cbtc_tooltip = '';
            }

            $html .= sprintf('%1$s', $cbtc_tooltip);

            $html .= $preview_html;

            //$html .= sprintf( '<span class="description"> %s</span>', $args['desc'] );
            echo $html;
        }

        /**
         * Displays a password field for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_password($args) {

            $value = esc_attr($this->get_option($args['id'], $args['section'], $args['std']));
            $size  = isset($args['size']) && !is_null($args['size']) ? $args['size'] : 'regular';

            $html = sprintf('<input type="password" style ="height:30px;" class="%1$s-text" id="%2$s[%3$s]" name="%2$s[%3$s]" value="%4$s"/>', $size, $args['section'], $args['id'], $value);
            $html .= sprintf('<span class="description"> %s</span>', $args['desc']);

            echo $html;
        }

        /**
         * Displays a color picker field for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_color($args) {

            $value = esc_attr($this->get_option($args['id'], $args['section'], $args['std']));
            $size  = isset($args['size']) && !is_null($args['size']) ? $args['size'] : 'regular';

            $html = sprintf('<input type="text" style="height: 30px;" class="%1$s-text wp-color-picker-field" id="%2$s[%3$s]" name="%2$s[%3$s]" value="%4$s" data-default-color="%5$s" />', $size, $args['section'], $args['id'], $value, $args['std']);
            $html .= sprintf('<span class="description" style="display:block;"> %s</span>', $args['desc']);

            echo $html;
        }

        /**
         * @param $args
         * A date picker field added by codeboxr
         */
        function callback_datepicker($args) {

            $value = esc_attr($this->get_option($args['id'], $args['section'], $args['std']));
            $size  = isset($args['size']) && !is_null($args['size']) ? $args['size'] : 'regular';
            $html  = sprintf('<div class="">
                          <div  class="input-append" >
                            <input data-format="dd/MM/yyyy hh:mm:ss" id="cb_custom_datetimepicker" style="height: 30px;" type="text" class="%1$s-text" id="%2$s[%3$s]" name="%2$s[%3$s]" value="%4$s"  />
                                <span class="add-on">
                                  <i class="fa fa-calendar" data-time-icon="icon-time" data-date-icon="icon-calendar">
                                  </i>
                                </span>
                              </div>
                            </div>
                          ', $size, $args['section'], $args['id'], $value);
            $html .= sprintf('<span class="description" style="display:block;"> %s</span>', $args['desc']);

            echo $html;
        }

        /**
         * @param $args
         * A timepicker field add by codeboxr use jquery ui
         */
        function callback_timepicker($args) {
            $value = esc_attr($this->get_option($args['id'], $args['section'], $args['std']));
            $size  = isset($args['size']) && !is_null($args['size']) ? $args['size'] : 'regular';
            $html  = sprintf('<div class="">
                            <div  class="input-append" >
                            <input data-format="hh:mm:ss" id="cb_custom_timepicker" type="text" style="height: 30px;" class="%1$s-text" id="%2$s[%3$s]" name="%2$s[%3$s]" value="%4$s"  />
                                <span class="add-on">
                                  <i data-time-icon="icon-time" class="fa fa-clock-o" data-date-icon="icon-calendar">
                                  </i>
                                </span>
                              </div>
                            </div>
                          ', $size, $args['section'], $args['id'], $value);
            $html .= sprintf('<span class="description" style="display:block;"> %s</span>', $args['desc']);

            echo $html;
        }

        /**
         * Sanitize callback for Settings API
         */
        function sanitize_options($options) {
            $options = is_array($options) ? $options : array();

            foreach ($options as $option_slug => $option_value) {
                $sanitize_callback = $this->get_sanitize_callback($option_slug);

                // If callback is set, call it
                if ($sanitize_callback) {
                    $options[$option_slug] = call_user_func($sanitize_callback, $option_value);
                    continue;
                }

                // Treat everything that's not an array as a string
                if (!is_array($option_value)) {
                    $options[$option_slug] = sanitize_text_field($option_value);
                    continue;
                }
            }
            //  var_dump($options);
            return $options;
        }

        /**
         * Get sanitization callback for given option slug
         *
         * @param string $slug option slug
         *
         * @return mixed string or bool false
         */
        function get_sanitize_callback($slug = '') {
            if (empty($slug))
                return false;
            // Iterate over registered fields and see if we can find proper callback
            foreach ($this->settings_fields as $section => $options) {
                foreach ($options as $option) {
                    if ($option['name'] != $slug)
                        continue;
                    // Return the callback name
                    return isset($option['sanitize_callback']) && is_callable($option['sanitize_callback']) ? $option['sanitize_callback'] : false;
                }
            }
            return false;
        }

        /**
         * Get the value of a settings field
         *
         * @param string  $option  settings field name
         * @param string  $section the section name this field belongs to
         * @param string  $default default text if it's not found
         * @return string
         */
        function get_option($option, $section, $default = '') {

            $options = get_option($section);
            // var_dump($options);
            if (isset($options[$option])) {

                return $options[$option];
            }

            return $default;
        }

        /**
         * Show navigations as tab
         *
         * Shows all the settings section labels as tab
         */
        function show_navigation() {
            $html = '<h2 class="nav-tab-wrapper">';

            foreach ($this->settings_sections as $tab) {
                if (!array_key_exists('only_in_meta', $tab)) {
                    $html .= sprintf('<a href="#%1$s" class="nav-tab cbtwittercard-nav-tab" id="%1$s-tab">%2$s</a>', $tab['id'], $tab['title']);
                }
            }

            $html .= '</h2>';

            echo $html;
        }

        /**
         * Show the section settings forms
         *
         * This function displays every sections in a different form
         */
        function show_forms() {
            ?>
            <div class="metabox-holder">
                <div class="postbox">
            <?php foreach ($this->settings_sections as $form) { ?>
                <?php if (!array_key_exists('only_in_meta', $form)): ?>
                            <div id="<?php echo $form['id']; ?>" class="cbtwittercard-group">
                                <form method="post" action="options.php">

                    <?php do_action('wsa_form_top_' . $form['id'], $form); ?>
                    <?php settings_fields($form['id']); ?>
                    <?php do_settings_sections($form['id']); ?>
                    <?php do_action('wsa_form_bottom_' . $form['id'], $form); ?>

                                    <div style="padding-left: 10px">
                    <?php submit_button(); ?>
                                    </div>
                                </form>
                            </div>
                <?php endif;
            }
            ?>
                </div>
            </div>
            <?php
            $this->script();
        }

        /**
         * Tabbable JavaScript codes & Initiate Color Picker
         *
         * This code uses localstorage for displaying active tabs
         */
        function script() {
            ?>

                    <?php
                }

            }

            
endif;
